import java.util.*;
/**
 * Klasse um Produkte die aus Einzelteilen und Unterprodukten bestehen zu
 * modelieren.
 */
public class Produkt {
	/* Ein Produkt besitzen einen Namen und einen Pries */
	private String name;
	private double price;
	/* Sie besteht entweder aus einem Einzelteil */
	private boolean working;
	private boolean singlePart;
	/* oder aus einem oder mehreren Unterprodukten */
	private LinkedList Components;
	private boolean multiPart;
	/**
	 * Erzeuge ein neues Produkt ohne Bestandteile
	 * @param name Name des Produktes
	 */
	public Produkt(String name) {
		this.name = name;
		price = 0.0;
		singlePart = false;
		multiPart = false;
		Components = new LinkedList();
	}
	/**
	 * baue ein Produkt aus einem Einzelteil
	 * @param name Name des Teils
	 * @param price Preise in Euro
	 * @param working ist das Teil funktionsf&auml;hig
	 * @return 0 beim erfolgreichen Anlegen, -1 wenn das Produkt aus
	 * Unterprodukten besteht und daher nicht aus einem Einzelteil
	 * aufgebaut werden kann.
	 */
	public int buildFromPart(String name, double price, boolean working){
		if (multiPart == true) {
			return(-1);
		} else {
			this.name = name;
			this.price = price;
			singlePart = true;
			this.working = working;
			return(0);
		}
	}
	/**
	 * f&uuml;ge einem Produkt ein neues Unterprodukt hinzu
	 * @param sub neues Unterprodukt
	 * @return 0 beim erfolgreichen Hinzuf&uuml;gen, -1 wenn das Produkt
	 * aus einem Einzelteil besteht und daher keine zus&auml;tzlichen
	 * Unterprodukte enthalten kann
	 */
	public int addComponent(Produkt sub) {
		if (singlePart == true) {
			return(-1);
		} else {
			multiPart = true;
			Components.add(sub);
			return(0);
		}
	}
	/**
	 * liefert den Preis einens Produktes zur&uuml;ck
	 * @return den Preise des Einzelteiles bzw. die Summe aller Preise
	 * der Teilproukte zuz&uuml;glich 10% Montagekosten.
	 */
	public double getPrice() {
		if (singlePart == true) {
			return(price);
		} else if (multiPart == true) {
			double summe = 0.0;
			ListIterator li = Components.listIterator(0);
			while(li.hasNext()) {
				Produkt next = (Produkt)li.next();
				summe += next.getPrice();
			}
			return(1.1 * summe);
		} else {
			return(0.0);
		}
	}
	/**
	 * liefert den Zustand einen Produktes
	 * @return sind _alle_ Beszandteile heil
	 */
	public boolean isWorking() {
		if (singlePart == true) {
			return(working);
		} else if (multiPart == true) {
			ListIterator li = Components.listIterator(0);
			while(li.hasNext()) {
				Produkt next = (Produkt)li.next();
				if (next.isWorking() != true) {
					return(false);
				}
			}
			return(true);
		} else {
			return(false);
		}
	}
}
